@@ -1,7 +1,20 @@
+# Copyright 2020 ADTRAN, Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
 import os
 import ssl
 import six
 import json
+import time
 import uuid
 import kafka
 import getpass
@@ -11,6 +24,7 @@ import tempfile
 import threading
 import pkg_resources
 
+from datetime import datetime
 import robot
 from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError
 from robot.utils import robottime
@@ -76,14 +90,22 @@ class KafkaListener(threading.Thread):
     Internal helper class used by this client.
     """
 
-    def __init__(self, topic_name, callback, consumer):
+    def __init__(self, topic_name, callback, consumer, **kwargs):
+        """
+        - topic_name: The Kafka topic to be listened to
+        - callback: The function to be executed when Kafka message arrives
+        - consumer: The Kafka consumer object of type kafka.KafkaConsumer()
+        - **kwargs: The keyword arguments:
+            - timestamp_from: To be used in the callback function, description in KafkaClient.subscribe()
+        """
         super(KafkaListener, self).__init__()
 
         self.setDaemon(True)
 
         self._topic = topic_name
-        self._cb = callback
+        self._cb_func = callback
         self._consumer = consumer
+        self._kwargs = kwargs
         self.id = str(uuid.uuid4()).replace('-', '')
 
         self.error = []
@@ -93,7 +115,7 @@ class KafkaListener(threading.Thread):
             self._consumer.subscribe([self._topic])
             for record in self._consumer:
                 try:
-                    self._cb(KafkaRecord(record))
+                    self._cb_func(KafkaRecord(record), **self._kwargs)
                 except Exception as e:
                     logging.error('Error while deliver message: %s' % e)
                     self.stop()
@@ -146,7 +168,7 @@ class KafkaClient(object):
         self._consumer_config = kafka.KafkaConsumer.DEFAULT_CONFIG
         self._topic_name = None
 
-        self._enable_logging(kwargs.get('root_logging', True))
+        self._enable_logging(kwargs.get('root_logging', False))
         logging.getLogger('kafka').propagate = False
         self._log = logging.getLogger('kafka.conn')
 
@@ -262,16 +284,21 @@ class KafkaClient(object):
             logging_dict['loggers'].update({'': {'level': 'NOTSET'}})
         logging.config.dictConfig(logging_dict)
 
-    def connection_open(self, host, port='9093', topic_name=''):
+    def connection_open(self, host, port='9093', topic_name='', **kwargs):
         """
         Opens a connection to the Kafka host.
 
-
         *Parameters*:
         - host: <string>|<IP address>; Name or IP address of the Kafka host.
         - port: <number>; TCP port of the Kafka host. Default: 9093.
         - topic_name: <string>; The name of the topic to listen on; optional. If not set, the keyword _subscribe_ can be used to set the subscription.
 
+        *Named parameters*:
+        - timestamp_from: <time string>, 0: Timestamp string format YYYY-MM-DD hh:mm:ss
+            (e.g. 2021-01-31 23:34:45) to define the time from which kafka records timestamps are collected in the
+            record list. If 0 then the filtering is switched off and received Kafka records are collected. If
+            _timestamp_from_ is not set then the library will use the current time for the filter.
+
         To set connection parameters such as certificates or authentication methods use _configuration_set_, e.g. for configuring ssl use
 
         | Configuration Set | ssl_cafile=nms_cachain.crt | ssl_certfile=nms_suite.crt | ssl_keyfile=nms_suite.key | ssl_check_hostname=${False} |
@@ -284,7 +311,7 @@ class KafkaClient(object):
         self.configuration_set(bootstrap_servers='%s:%s' % (self._host, int(self._port)))
 
         if topic_name:
-            return self.subscribe(topic_name=topic_name)
+            return self.subscribe(topic_name=topic_name, **kwargs)
 
     def connection_close(self):
         """
@@ -297,7 +324,28 @@ class KafkaClient(object):
         self._port = None
         self._cert_path = None
 
-    def subscribe(self, topic_name):  # type: (str) -> str
+    @staticmethod
+    def _get_timestamp(timestamp):
+
+        if timestamp is None or timestamp == '':
+            timestamp = datetime.now()
+
+        try:
+            timestamp = int(time.mktime(datetime.strptime(timestamp, '%Y-%m-%d %H:%M:%S').timetuple()) * 1000)
+        except TypeError:
+            try:
+                timestamp = int(time.mktime(timestamp.timetuple()) * 1000)
+            except AttributeError:
+                timestamp = int(timestamp)
+        except ValueError:
+            try:
+                timestamp = int(timestamp)
+            except ValueError:
+                raise ValueError('"%s" is not a valid input: should be of format "YYYY-MM-DD hh:mm:ss"' % timestamp)
+
+        return timestamp
+
+    def subscribe(self, topic_name, **kwargs):  # type: (str) -> str
         """
         Subscribe for an external topic and starts a listener on this topic. All records received in this listener are
         stored in a record list, which can be retrieved with keyword _records_get_. Records are recorded until it is
@@ -306,10 +354,19 @@ class KafkaClient(object):
         *Parameters*:
         - topic_name: <string>; The name of the topic to listen on.
 
+        *Named parameters*:
+        - timestamp_from: <time string>, 0: Timestamp string format YYYY-MM-DD hh:mm:ss
+            (e.g. 2021-01-31 23:34:45) to define the time from which kafka records timestamps are collected in the
+            record list. If 0 then the filtering is switched off and received Kafka records are collected. If
+            _timestamp_from_ is not set then the library will use the current time for the filter.
+        - human_readable_timestamps:
+
         *Return*: The ID of the subscription. This ID can be used to unsubscribe.
         """
+        kwargs['timestamp_from'] = self._get_timestamp(kwargs.get('timestamp_from'))
+
         consumer = self._create_consumer()
-        listener = KafkaListener(topic_name=topic_name, callback=self._add_record, consumer=consumer)  # type: KafkaListener
+        listener = KafkaListener(topic_name=topic_name, callback=self._add_record, consumer=consumer, **kwargs)  # type: KafkaListener
         self._listeners[listener.id] = listener
         listener.start()
         return listener.id
@@ -374,7 +431,7 @@ class KafkaClient(object):
 
         *Return*: A list of dictionaries. Each dictionary represents a record and consists of keys:
         - topic: Name of the topic
-        - timestamp: Number of milliseconds since Jan 01 1970, midnight (UTC)
+        - timestamp: Number of milliseconds since Jan 01 1970, midnight (UTC) as string format "yyyy-mm-dd hh:MM:ss.ffffff"
         - message: Message content as dictionary
         """
         if not topic_name or topic_name == 'all':
@@ -396,7 +453,7 @@ class KafkaClient(object):
 
     def configuration_set(self, **kwargs):
         """
-        Available setting with example values. To get actual used setting use _configuration_get_.
+        Available setting with example values. To get actual used setting use _Configuration Get_.
         Values must be supplied using there correct type (e.g. int, str, bool)
 
         - 'bootstrap_servers': 'localhost',
@@ -460,20 +517,31 @@ class KafkaClient(object):
                 self._consumer_config['ssl_context'] = None
 
     def configuration_get(self):
+        """
+        Shows the current setting of the Kafka interface. For available keys check keyword _Configuration Set_
+        """
         return self._consumer_config
 
-    def _add_record(self, record):
+    def _add_record(self, record, **kwargs):
         try:
-            record = {
-                'topic': record.topic,
-                'timestamp': record.timestamp,
-                'message': record.value
-            }
+
+            print('filter timestamp %s, timestamp of Kafka record %s' % (kwargs.get('timestamp_from'), record.timestamp))
+            if kwargs.get('timestamp_from') == 0 or kwargs.get('timestamp_from') <= record.timestamp:
+
+                record = {
+                    'topic': record.topic,
+                    'timestamp': datetime.fromtimestamp(int(record.timestamp) / 1e3).strftime('%Y-%m-%d %H:%M:%S.%f'),
+                    'timestamp_sec': record.timestamp,
+                    'message': record.value
+                }
+
+                if record not in self._record_list:
+                    self._record_list.append(record)
+
+                print('Kafka record inserted in list')
+
         except Exception as e:
             self._log.error(e)
 
-        if record not in self._record_list:
-            self._record_list.append(record)
-
     def _create_consumer(self):
         return kafka.KafkaConsumer(**self._consumer_config)
